package at.xaxa.ledger.data

import android.util.Log
import at.xaxa.ledger.data.db.BigEntity.BigEntity
import at.xaxa.ledger.data.db.Category.CategoryEntity
import at.xaxa.ledger.data.db.Entry.EntryEntity
import at.xaxa.ledger.data.db.LedgerDao
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.map


class EntryRepository(private val ledgerDao: LedgerDao){


    fun getAllBigEntries(): Flow<List<BigEntity>> {
        return ledgerDao.getAllBigEntries().map {
            it.map {entry -> BigEntity(entry.entryId, entry.name, entry.amount, entry.date, entry.categoryID, entry.categoryName, entry.icon) }
        }
    }

    fun getAllCategories(): Flow<List<CategoryEntity>> {
        return ledgerDao.getAllCategory().map {
            it.map {category -> CategoryEntity(category._id, category.categoryName, category.icon) }
        }
    }
    suspend fun findEntryByCategoryId(id: Int): Int {
        return ledgerDao.findEntryByCategoryId(id)
    }

    suspend fun calculateBalance(): Float{
        return ledgerDao.calculateBalance()
    }

    suspend fun findEntryById(id: Int): Entry {
        val entry = ledgerDao.findEntryById(id)
        return Entry(
            entry.entryId, entry.name, entry.amount, entry.date, entry.categoryID
        )
    }
    suspend fun findCategoryById(id: Int): CategoryEntity {
        val category = ledgerDao.findCategoryById(id)
        return CategoryEntity(
            category._id, category.categoryName, category.icon
        )
    }
    suspend fun insertEntry(entry: Entry) {
        ledgerDao.addEntry(EntryEntity(entryId=0, entry.name, entry.amount, entry.date, entry.categoryID))
    }
    suspend fun insertCategory(category: CategoryEntity) {
        ledgerDao.addCategory(CategoryEntity(_id=0, category.categoryName, category.icon))
    }

    suspend fun updateEntry(entry: Entry) {
        ledgerDao.updateEntry(EntryEntity(entry.id, entry.name, entry.amount, entry.date, entry.categoryID))
    }
    suspend fun updateCategory(category: CategoryEntity) {
        ledgerDao.updateCategory(CategoryEntity(category._id, category.categoryName, category.icon))
    }

    suspend fun deleteEntry(entry: Entry) {
        ledgerDao.deleteEntry(EntryEntity(entryId = entry.id, entry.name, entry.amount, entry.date, entry.categoryID))
    }
    suspend fun deleteCategory(category: CategoryEntity) {
        ledgerDao.deleteCategory(CategoryEntity(_id = category._id, category.categoryName, category.icon))
    }

}