package at.xaxa.ledger.ui.category.edit

import android.util.Log
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import at.xaxa.ledger.data.EntryRepository
import at.xaxa.ledger.data.db.Category.CategoryEntity
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

data class CategoryUIState(val category: CategoryEntity = CategoryEntity(0,"",0))



class EditCategoryViewModel(private val savedStateHandle: SavedStateHandle, private val entryRepository: EntryRepository) : ViewModel() {
    private val categoryId: Int = checkNotNull(savedStateHandle["categoryId"])

    var categoryUi by mutableStateOf(CategoryUIState())
        private set
    var deletePossible by mutableStateOf(false)
    var deleteStarted by mutableStateOf(false)

    private var _redirect = MutableStateFlow(false)
    val redirect: StateFlow<Boolean> get() = _redirect

    init {
        viewModelScope.launch {
            val category = withContext(Dispatchers.IO) {
                entryRepository.findCategoryById(categoryId)
            }
            categoryUi = CategoryUIState(category)

            _redirect.value = false;
        }
    }

    fun updateCategory(category: CategoryEntity) {
        categoryUi = categoryUi.copy(category = category)
    }

    fun deleteCategory(){
        viewModelScope.launch {
            entryRepository.deleteCategory(categoryUi.category)
        }
    }

    fun saveCategory() {
        viewModelScope.launch {
            entryRepository.updateCategory(categoryUi.category)
        }
    }

    fun findEntryByCategoryId() {
        viewModelScope.launch {
            val fetchedEntries = withContext(Dispatchers.IO) {
              entryRepository.findEntryByCategoryId(categoryId)
            }

            if(fetchedEntries>0){
                deletePossible = true
            }else{
                Log.w("lolo", "findEntryByCategoryId: "+fetchedEntries, )
                deleteCategory()
                _redirect.value= true;
            }
        }
    }
}
